/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2024/08/22
 * Description        : Main program body.
 *********************************************************************************
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * Attention: This software (modified or not) and binary are used for 
 * microcontroller manufactured by Nanjing Qinheng Microelectronics.
 *******************************************************************************/

/*
 *@Note
 *TIMER_INT
 *MODE 1 Timer count triggers update interrupt. Updata_time = psc*arr*RepetitionCounter/system
 *MODE 2 generates an update event and triggers an update interrupt.
 *
 */

#include "debug.h"

#include "debug.h"

/* Mode Definition */
#define MODE1   0
#define MODE2   1

/* Mode Selection */
#define MODE MODE1
//#define MODE MODE2

/*********************************************************************
 * @fn      TIM1_UP_IRQHandler
 *
 * @brief   This function handles TIM1 UP exception.
 *
 *
 * @return  none
 */
void TIM1_UP_IRQHandler(void) __attribute__((interrupt("WCH-Interrupt-fast")));
void TIM2_UP_IRQHandler(void) __attribute__((interrupt("WCH-Interrupt-fast")));
void TIM3_UP_IRQHandler(void) __attribute__((interrupt("WCH-Interrupt-fast")));


void TIM1_UP_IRQHandler(void)
{
    if(TIM_GetITStatus(TIM1, TIM_IT_Update)==SET)
    {
        printf("--------updata1\r\n");
    }
    TIM_ClearITPendingBit( TIM1, TIM_IT_Update );
}

void TIM2_UP_IRQHandler(void)
{
    if(TIM_GetITStatus(TIM2, TIM_IT_Update)==SET)
    {//TIM2->CNT = 0;
        //TIM2->DMAINTENR &= ~TIM_IT_Update;
        printf("--------updata2\r\n");
    }
    TIM_ClearITPendingBit( TIM2, TIM_IT_Update );
}

void TIM3_UP_IRQHandler(void)
{
    if(TIM_GetITStatus(TIM3, TIM_IT_Update)==SET)
    {//TIM2->CNT = 0;
        //TIM2->DMAINTENR &= ~TIM_IT_Update;
        printf("--------updata3\r\n");
    }
    TIM_ClearITPendingBit( TIM3, TIM_IT_Update );
}
/*********************************************************************
 * @fn      TIM1_INT_Init
 *
 * @brief   Initializes TIM1 output compare.
 *
 * @param   arr - the period value.
 *          psc - the prescaler value.
 *
 * @return  none
 */
void TIM1_INT_Init( u16 arr, u16 psc)//TIM1_INT_Init( 100-1, 48000-1);
{

    NVIC_InitTypeDef NVIC_InitStructure={0};
    TIM_TimeBaseInitTypeDef TIM_TimeBaseInitStructure={0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1, ENABLE );

    TIM_TimeBaseInitStructure.TIM_Period = arr; //arr = 100,即n=100，自动重装载值,t = n/f，即100ms
    TIM_TimeBaseInitStructure.TIM_Prescaler = psc;//psc = 48000,预分频为psc-1,1KHZ
    TIM_TimeBaseInitStructure.TIM_ClockDivision = TIM_CKD_DIV1;
    TIM_TimeBaseInitStructure.TIM_CounterMode = TIM_CounterMode_Up;
    TIM_TimeBaseInitStructure.TIM_RepetitionCounter = 10;//重复计数，重复计数的作用，10x100ms，定时器的更新事件（Update Event）每发生50次才会触发一次主更新事件，即，要5s才
    TIM_TimeBaseInit( TIM1, &TIM_TimeBaseInitStructure);//若重复计数为0，则100ms触发一次主更新事件
 

    TIM_ClearITPendingBit( TIM1, TIM_IT_Update );

    NVIC_InitStructure.NVIC_IRQChannel =TIM1_UP_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority =0;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority =1;
    NVIC_InitStructure.NVIC_IRQChannelCmd =ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    TIM_ITConfig(TIM1, TIM_IT_Update, ENABLE);

}

/*********************************************************************
 * @fn      TIM1_INT_Init
 *
 * @brief   Initializes TIM1 output compare.
 *
 * @param   arr - the period value.
 *          psc - the prescaler value.
 *
 * @return  none
 */
void TIM2_INT_Init( u16 arr, u16 psc)//TIM1_INT_Init( 100-1, 48000-1);
{

    NVIC_InitTypeDef NVIC_InitStructure={0};
    TIM_TimeBaseInitTypeDef TIM_TimeBaseInitStructure={0};

    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE );

    TIM_TimeBaseInitStructure.TIM_Period = arr; //arr = 100,即n=100，自动重装载值,t = n/f，即100ms
    TIM_TimeBaseInitStructure.TIM_Prescaler = psc;//psc = 48000,预分频为psc-1,1KHZ
    TIM_TimeBaseInitStructure.TIM_ClockDivision = TIM_CKD_DIV1;
    TIM_TimeBaseInitStructure.TIM_CounterMode = TIM_CounterMode_Up;
    TIM_TimeBaseInitStructure.TIM_RepetitionCounter = 10;//重复计数，重复计数的作用，10x100ms，定时器的更新事件（Update Event）每发生10次才会触发一次主更新事件，即，要1s才
    TIM_TimeBaseInit( TIM2, &TIM_TimeBaseInitStructure);//若重复计数为0，则100ms触发一次主更新事件

    TIM_ClearITPendingBit( TIM2, TIM_IT_Update );

    NVIC_InitStructure.NVIC_IRQChannel =TIM2_UP_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority =0;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority =2;
    NVIC_InitStructure.NVIC_IRQChannelCmd =ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);

}


void TIM3_INT_Init( u16 arr, u16 psc)//TIM1_INT_Init( 100-1, 48000-1);
{

    NVIC_InitTypeDef NVIC_InitStructure={0};
    TIM_TimeBaseInitTypeDef TIM_TimeBaseInitStructure={0};

    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE );

    TIM_TimeBaseInitStructure.TIM_Period = arr; //arr = 100,即n=100，自动重装载值,t = n/f，即100ms
    TIM_TimeBaseInitStructure.TIM_Prescaler = psc;//psc = 48000,预分频为psc-1,1KHZ
    TIM_TimeBaseInitStructure.TIM_ClockDivision = TIM_CKD_DIV1;
    TIM_TimeBaseInitStructure.TIM_CounterMode = TIM_CounterMode_Up;
    //TIM_TimeBaseInitStructure.TIM_RepetitionCounter = 10;//对定时器3无效因此注释掉
    TIM_TimeBaseInit( TIM3, &TIM_TimeBaseInitStructure);


    TIM_ClearITPendingBit( TIM3, TIM_IT_Update );

    //可能是下面这几句程序初始化失败
    NVIC_InitStructure.NVIC_IRQChannel =TIM3_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority =0;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority =7;
    NVIC_InitStructure.NVIC_IRQChannelCmd =ENABLE;
    NVIC_Init(&NVIC_InitStructure);

    TIM_ITConfig(TIM3, TIM_IT_Update, ENABLE);



}



/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
    SystemCoreClockUpdate();
    Delay_Init();
    // 初始化GPIO
    GPIO_Button_Init();
#if (SDI_PRINT == SDI_PR_OPEN)
    SDI_Printf_Enable();
#else
    USART_Printf_Init(115200);
#endif
    printf("SystemClk:%d\r\n",SystemCoreClock);
    printf( "ChipID:%08x\r\n", DBGMCU_GetCHIPID() );

   TIM1_INT_Init( 100-1, 48000-1);
   TIM2_INT_Init( 100-1, 48000-1);
   TIM3_INT_Init( 1000-1, 48000-1);
#if(MODE==MODE1)

    TIM_Cmd( TIM1, ENABLE );
    TIM_Cmd( TIM2, ENABLE );
    TIM_Cmd( TIM3, ENABLE );//会卡在这里
   
    Delay_Ms(5000);//测试
    printf("INit...\r\n");

#elif(MODE==MODE2)

    TIM_GenerateEvent(TIM1, TIM_IT_Update);

#endif

    while(1)
    {

    }
}
